# django_clite

# CLI configuration

CLI_NAME_KEY = "django-clite"
DJANGO_FILES_KEY = "django_files"
ENABLE_DEBUG_KEY = "enable_debug"
ENABLE_DRY_RUN_KEY = "enable_dry"
ENABLE_FORCE_KEY = "GENY_ENABLE_FORCE"
ENABLE_VERBOSITY_KEY = "enable_verbosity"

TEMPLATES_KEY = "templates"
TEMPLATES_DIRECTORY_ENV_VAR = "DJANGO_CLITE_TEMPLATES_DIR"
PLUGINS_ENV_VAR = "DJANGO_CLITE_PLUGINS"

DEV_MODE_ENV_VAR = "DJANGO_CLITE_DEV_MODE"

# Project information

APPLICATION_NAME_KEY = "app"
PROJECT_NAME_KEY = "project"



import click
import inflection

from pathlib import Path
from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import RemoveLineFromFile
from django_clite.core.logger import logger
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands import command_defaults
from django_clite.commands.callbacks import sanitized_string_callback


SUPPORTED_SCOPES = [
    "model",
    "viewset",
]


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.option("--scope", required=True, type=click.Choice(SUPPORTED_SCOPES))
@click.option("--full", is_flag=True, help=f"Destroy tests for {SUPPORTED_SCOPES}")
@click.pass_context
def test(ctx, name, scope, full):
    """
    Destroy TestCases.
    """

    if scope and full:
        logger.error("Flags --scope and --full cannot be used simultaneously.")
        raise click.Abort()

    scopes = SUPPORTED_SCOPES if full else [scope]

    for s in scopes:
        file = File(name=f"tests/{inflection.pluralize(s)}/{name}_test.py")

        file.destroy(
            after_hooks=[
                RemoveLineFromFile(
                    Path(f"tests/{inflection.pluralize(s)}/__init__.py"),
                    command_defaults.test(name),
                ),
            ],
            **ctx.obj,
        )



import click
import pathlib
import inflection

from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import AddLineToFile, TouchFile
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands import command_defaults
from django_clite.commands.callbacks import sanitized_string_callback


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.option("--read-only", is_flag=True, help="Create a read-only viewset.")
@click.option("--full", is_flag=True, help="Create related files (i.e. TestCases)")
@click.option(
    "--skip-import",
    is_flag=True,
    default=False,
    help="Do not import in __init__ module",
)
@click.pass_context
def viewset(ctx, name, read_only, full, skip_import):
    """
    Generate a viewset for a serializable model.
    """

    file = File(
        name=f"viewsets/{name}.py",
        template="viewsets/viewset.tpl",
        context={
            "name": name,
            "module": name,
            "read_only": read_only,
            "namespace": inflection.pluralize(name),
            "classname": inflection.camelize(name),
        },
    )

    # TODO: Create router file
    # router = Directory(
    #     "router",
    #     children=[
    #         File(name="__init__.py", template="app/router_init.tpl"),
    #         File(name="api.py", template="app/router_api.tpl"),
    #         File(name="router.py", template="app/router.tpl"),
    #     ],
    # )
    #
    # router.create(**ctx.obj)

    after_hooks = [TouchFile("viewsets/__init__.py")]

    if not skip_import:
        after_hooks.append(
            AddLineToFile(
                pathlib.Path("viewsets/__init__.py"),
                command_defaults.viewset(name),
                prevent_duplicates=True,
            )
        )

    file.create(after_hooks=after_hooks, **ctx.obj)

    if full:
        from .tests import test as cmd

        ctx.invoke(cmd, name=name, scope="viewset", skip_import=skip_import)



# django_clite:commands:new:defaults:project
from geny.core.filesystem.files import File
from geny.core.filesystem.directories import Directory
from geny.core.filesystem.transformations import MoveFile

project_transformations = []


def new_project(name: str, **context) -> Directory:
    project_transformations.append(
        MoveFile("settings.py", "settings/__init__.py"),
    )

    proj = Directory(
        name=name,
        children=[
            Directory(
                name="settings",
                children=[
                    File(name="api.py", template="project/api.tpl"),
                ],
            ),
            File(name="constants.py", content="# your constants go here"),
        ],
    )

    root = Directory(
        name=name,
        children=[
            proj,
            Directory(name="staticfiles"),
            Directory(
                name="templates",
                children=[
                    File(name="404.html", template="app/404.tpl"),
                    File(name="500.html", template="app/500.tpl"),
                ],
            ),
            File(name=".env", template="project/env.tpl"),
            File(name=".gitignore", template="project/gitignore.tpl"),
            File(name="README.md", template="project/README.tpl"),
            File(name="requirements.txt", template="project/requirements.tpl"),
        ],
    )

    # Handle user options

    if context.get("github", False):
        github = Directory(
            ".github",
            children=[
                Directory(
                    ".github",
                    children=[
                        File(name="ci.yml", template="github/ci.tpl"),
                        File(name="pull_request_template.md", template="github/pull_request_template.tpl"),
                    ],
                )
            ],
        )

        root.add_children(github) # noqa

    if context.get("docker", False):
        docker = [
            File(name=".dockerignore", template="docker/dockerignore.tpl"),
            File(name="Dockerfile", template="docker/dockerfile.tpl"),
            File(name="docker-compose.yml", template="docker/docker-compose.tpl"),
            File(name="docker-entrypoint.sh", template="docker/docker-entrypoint.tpl"),
        ]

        root.add_children(docker) # noqa

    if context.get("kubernetes", False):
        kubernetes = [
            Directory(
                ".kubernetes",
                children=[
                    File(name="deployment.yaml", template="kubernetes/deployment.tpl"),
                    File(name="service.yaml", template="kubernetes/service.tpl"),
                    File(name="ingress.yaml", template="kubernetes/ingress.tpl"),
                    File(name="configmap.yaml", template="kubernetes/configmap.tpl"),
                ],
            ),
        ]

        root.add_children(kubernetes) # noqa

    # TODO: implement celery option
    # if options.get("celery", False):

    # TODO: implement drf option
    # if options.get("drf", False):

    # TODO: implement dokku option
    # if options.get("dokku", False):

    # TODO: implement heroku option
    # if options.get("heroku", False):

    return root



# coding: utf-8
"""
    django-clite: a cli tool that handles creating and managing Django projects
"""
import os
from django_clite.constants import PLUGINS_ENV_VAR

__license__ = "BSD 3-Clause"
__author__ = "Leo Neto"
__copyright__ = "Copyright 2019-2023 Leo Neto"

COMMANDS_FOLDER = os.path.join(os.path.dirname(__file__), "commands")

PLUGINS_FOLDER = os.environ.get(PLUGINS_ENV_VAR, None)



# tests
import unittest

from pathlib import Path

from geny.core.templates.template import TemplateParser
from django_clite import template_files

temp_dir = Path(__file__).resolve().parent / "tmp"

parser = TemplateParser(
    templates_dir=[Path(template_files.__file__).resolve().parent],
    context={},
)

if __name__ == "__main__":
    unittest.main()



from click import Group


class AliasedGroup(Group):
    """
    Adds support for abbreviated commands
    """

    def get_command(self, ctx, cmd_name):
        rv = Group.get_command(self, ctx, cmd_name)

        if rv is not None:
            return rv

        matches = [
            match for match in self.list_commands(ctx) if match.startswith(cmd_name)
        ]

        if not matches:
            return None

        if len(matches) != 1:
            raise ctx.fail("Too many matches: %s" % ", ".join(sorted(matches)))

        return Group.get_command(self, ctx, matches[0])



import click
import pathlib
import inflection

from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import AddLineToFile, TouchFile

from django_clite.core.logger import logger
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands import command_defaults
from django_clite.commands.callbacks import sanitized_string_callback, fields_callback
from django_clite.constants import APPLICATION_NAME_KEY


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.argument("fields", nargs=-1, required=False, callback=fields_callback)
@click.option("-a", "--abstract", is_flag=True, help="Creates an abstract model type")
@click.option("--api", is_flag=True, help="Adds only related api resources")
@click.option("--full", is_flag=True, help="Adds all related resources")
@click.option("--admin", is_flag=True, help="Register admin model")
@click.option("--fixtures", is_flag=True, help="Create model fixture")
@click.option("--form", is_flag=True, help="Create model form")
@click.option("--serializers", is_flag=True, help="Create serializers")
# @click.option("--templates", is_flag=True, help="Create templates")
@click.option("--tests", is_flag=True, help="Create tests")
@click.option("--views", is_flag=True, help="Create views")
@click.option("--viewsets", is_flag=True, help="Create viewsets")
@click.option(
    "--skip-import",
    is_flag=True,
    default=False,
    help="Do not import in __init__ module",
)
@click.pass_context
def model(
    ctx,
    name,
    fields,
    abstract,
    api,
    full,
    admin,
    fixtures,
    form,
    serializers,
    # templates,
    tests,
    views,
    viewsets,
    skip_import,
):
    """
    Generates a model under the models directory.
    One can specify multiple attributes after the model's name, like so:

        django-clite g model track int:number char:title fk:album bool:is_favorite

    This will generate a Track model and add a foreign key of Album.
    If the model is to be added to admin.site one can optionally opt in by specifying the --register-admin flag.
    """

    if api and full:
        logger.error("Flags --api and --full cannot be used simultaneously.")
        raise click.Abort()

    current_app_scope = ctx.obj.get(APPLICATION_NAME_KEY)

    file = File(
        name=f"models/{name}.py",
        template="models/model.tpl",
        context={
            "api": api,
            "abstract": abstract,
            "classname": inflection.camelize(name),
            "fields": fields,
            "imports": dict((k, v) for k, v in fields.items() if v.is_relationship),
            "name": name,
            "table_name": f"{current_app_scope}.{inflection.pluralize(name)}",
        },
    )

    after_hooks = [TouchFile("models/__init__.py")]

    if not skip_import:
        after_hooks.append(
            AddLineToFile(
                pathlib.Path("models/__init__.py"),
                command_defaults.model(name),
                prevent_duplicates=True,
            )
        )

    file.create(after_hooks=after_hooks, **ctx.obj)

    def generate_related_resources():
        if admin or api or full:
            from .admin import admin as cmd

            ctx.invoke(cmd, name=name, fields=fields, skip_import=skip_import)

        if fixtures or api or full:
            from .fixtures import fixture as cmd

            ctx.invoke(cmd, model=name, fields=fields)

        if form or full:
            from .forms import form as cmd

            ctx.invoke(cmd, name=name, skip_import=skip_import)

        if serializers or api or full:
            from .serializers import serializer as cmd

            ctx.invoke(cmd, name=name, skip_import=skip_import)

        if tests or api or full:
            from .tests import test as cmd

            ctx.invoke(cmd, name=name, full=full)

        if views or full:
            from .views import view as cmd

            ctx.invoke(cmd, name=name, full=full, skip_import=skip_import)

        if viewsets or api or full:
            from .viewsets import viewset as cmd

            ctx.invoke(cmd, name=name, skip_import=skip_import)

    generate_related_resources()


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.argument("fields", nargs=-1, required=False, callback=fields_callback)
@click.option("--api", is_flag=True, help="Destroy only related api resources")
@click.pass_context
def scaffold(ctx, name, fields, api):
    """
    Generate all resources for a given model.
    """

    ctx.invoke(model, name=name, fields=fields, full=not api, api=api)



import re
import inflection


def inflect(noun, force_singular=False, force_plural=False):
    if force_plural == force_singular:
        return noun
    elif force_singular:
        return inflection.singularize(noun)
    elif force_plural:
        return inflection.pluralize(noun)

    return noun


def sanitized_string(text):
    r = inflection.transliterate(text.strip())
    r = re.sub(r"[:.,;\\/?!&$#@)(+=\'\"\-|\s]+", "_", r)
    r = r.strip("_")  # remove leading or trailing underscores
    return inflection.underscore(r).lower()



import click

from django_clite.commands.new.app import apps
from django_clite.commands.new.project import project


@click.group()
@click.pass_context
def new(ctx):
    """
    Create projects and apps.
    """

    ctx.ensure_object(dict)


[new.add_command(cmd) for cmd in [apps, project]]



import click

from geny.core.filesystem.files import File
from django_clite.commands.callbacks import sanitized_string_callback
from django_clite.decorators.scope import scoped, Scope


@scoped(to=Scope.APP)
@click.command()
@click.argument("model", required=True, callback=sanitized_string_callback)
@click.pass_context
def fixture(ctx, model):
    """
    Destroy model fixtures.
    """

    File(name=f"fixtures/{model}.json").destroy(**ctx.obj)



import click

from pathlib import Path
from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import RemoveLineFromFile
from django_clite.decorators.scope import scoped, Scope
from .. import command_defaults
from django_clite.commands.callbacks import sanitized_string_callback
from .template import template


SUPPORTED_CLASSES = [
    "create",
    "detail",
    "list",
    "update",
]


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.option("--class_", type=click.Choice(SUPPORTED_CLASSES))
@click.option("--full", is_flag=True, help="Delete all CRUD views")
@click.option(
    "--keep-templates",
    is_flag=True,
    default=False,
    help="Destroy related templates.",
)
@click.pass_context
def view(ctx, name, class_, full, keep_templates):
    """
    Destroy a view function or class.
    """

    classes = SUPPORTED_CLASSES if full else [class_]

    for k in classes:
        File(name=f"views/{name}{'_' + k if k else ''}.py").destroy(
            after_hooks=[
                RemoveLineFromFile(
                    Path("views/__init__.py"),
                    command_defaults.view(name, klass=k),
                ),
            ],
            **ctx.obj,
        )

    if keep_templates:
        return

    for class_ in classes:
        ctx.invoke(template, name=name, class_=class_)



# django_clite:core:field_parser
import inflection

from dataclasses import dataclass
from geny.core.decorators.singleton import singleton


json_compatible_fields = {
    "BigIntegerField": "pyint",
    "BooleanField": "pybool",
    "CharField": "text",
    "DateField": "future_date",
    "DateTimeField": "iso8601",
    "DecimalField": "pydecimal",
    "EmailField": "safe_email",
    "GenericIPAddressField": "ipv4",
    "FileField": "file_path",
    "FilePathField": "file_path",
    "FloatField": "pyfloat",
    "ImageField": "image_url",
    "IntegerField": "pyint",
    "SlugField": "slug",
    "TextField": "text",
    "TimeField": "time",
    "URLField": "url",
    "UUIDField": "uuid4",
}


@dataclass
class FieldOptions:
    kind: str
    options: dict = None
    supports_admin: bool = True
    is_fk_relationship: bool = False
    is_many_relationship: bool = False

    @property
    def is_relationship(self) -> bool:
        return self.is_fk_relationship or self.is_many_relationship

    @property
    def is_media_field(self) -> bool:
        return self.kind in ["FileField", "FilePathField", "ImageField"]

    @classmethod
    def klass_name(cls, attr_name):
        return inflection.camelize(inflection.singularize(attr_name)).strip()

    @classmethod
    def module_name(cls, attr_name):
        return inflection.singularize(attr_name).strip()

    @classmethod
    def upload_path(cls, attr_name, model_name) -> str:
        if not cls.is_media_field:
            return ""

        return f"uploads/{inflection.pluralize(model_name.lower())}/{inflection.pluralize(attr_name)}/"

    @property
    def example_value(self):
        x = json_compatible_fields.get(self.kind, "")
        return x

    def field_options(self, attr_name: str, model_name: str):
        if self.options is None:
            self.options = {}

        if self.is_relationship:
            self.options.update(
                {
                    "related_name": f"'{inflection.pluralize(inflection.dasherize(model_name.lower()))}'"
                }
            )

        options = [self.klass_name(attr_name)] if self.is_relationship else []
        options.append(f"_('{attr_name}')")

        if self.options is None:
            return ""

        for k, v in self.options.items():
            options.append(f"{k}={v}")

        if self.is_media_field:
            options.append(f"upload_to='{self.upload_path(attr_name, model_name)}'")

        return ", ".join(options)


@singleton
class AttributeFactory:
    def __init__(self, associations: dict[str, FieldOptions], aliases: dict[str, str]):
        self.registry = associations
        self.aliases = aliases

    def field_options(self, kind: str, name: str) -> FieldOptions:
        if kind == "":
            return None

        options = self.registry.get(kind, None)

        if options is None:
            alias = self.aliases.get(kind, None)
            options = self.registry.get(alias, None)

        if options is None:
            return None

        return options

    def parsed_fields(self, values: list[str]) -> dict[str, FieldOptions]:
        pairs = dict(arg.split(":") for arg in values)

        fields = {}
        for n, k in pairs.items():
            f = self.field_options(k, n)
            fields[n] = f

        return fields


attribute_aliases = {
    "bigint": "big",
    "big-int": "big",
    "integer": "int",
    "boolean": "bool",
    "string": "text",
    "file-path": "filepath",
    "photo": "image",
    "ipaddress": "ip",
    "ip-address": "ip",
    "belongsto": "fk",
    "belongs-to": "fk",
    "foreignkey": "fk",
    "foreign-key": "fk",
    "one-to-one": "one",
    "hasone": "one",
    "has-one": "one",
    "many": "hasmany",
    "has-many": "hasmany",
    "manytomany": "hasmany",
    "many-to-many": "hasmany",
}

field_registry = {
    # Relationships
    "fk": FieldOptions(
        kind="ForeignKey",
        options={"blank": False, "on_delete": "models.DO_NOTHING"},
        is_fk_relationship=True,
    ),
    "one": FieldOptions(
        kind="OneToOneField",
        options={"blank": False, "on_delete": "models.CASCADE", "primary_key": True},
        is_fk_relationship=True,
    ),
    "hasmany": FieldOptions(
        kind="ManyToManyField",
        options={"blank": True, "on_delete": "models.DO_NOTHING"},
        supports_admin=False,
        is_many_relationship=True,
    ),
    # Boolean
    "bool": FieldOptions(kind="BooleanField", options={"default": False}),
    # Numeric
    "big": FieldOptions(kind="BigIntegerField"),
    "decimal": FieldOptions(
        kind="DecimalField", options={"decimal_places": 2, "max_digits": 8}
    ),
    "float": FieldOptions(kind="FloatField"),
    "int": FieldOptions(kind="IntegerField"),
    # Text
    "char": FieldOptions(kind="CharField", options={"max_length": 100, "blank": False}),
    "email": FieldOptions(kind="EmailField"),
    "slug": FieldOptions(kind="SlugField", options={"unique": True}),
    "text": FieldOptions(
        kind="TextField", options={"blank": False}, supports_admin=False
    ),
    "url": FieldOptions(kind="URLField"),
    "uuid": FieldOptions(
        kind="UUIDField", options={"default": "uuid.uuid4", "editable": False}
    ),
    # Files
    "file": FieldOptions(
        kind="FileField", options={"blank": False}, supports_admin=False
    ),
    "filepath": FieldOptions(
        kind="FilePathField", options={"blank": True}, supports_admin=False
    ),
    "image": FieldOptions(
        kind="ImageField", options={"blank": False}, supports_admin=False
    ),
    # Date
    "date": FieldOptions(kind="DateField", options={"auto_now": True}),
    "datetime": FieldOptions(kind="DateTimeField", options={"auto_now": True}),
    "duration": FieldOptions(kind="DurationField"),
    "time": FieldOptions(kind="TimeField", options={"auto_now": True}),
    # Ip
    "ip": FieldOptions(kind="GenericIPAddressField"),
}

# Singleton
AttributeFactory(field_registry, attribute_aliases)



import click

from pathlib import Path
from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import RemoveLineFromFile
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands import command_defaults
from django_clite.commands.callbacks import sanitized_string_callback


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", callback=sanitized_string_callback)
@click.pass_context
def admin(ctx, name):
    """
    Destroy an admin model.
    """

    File(name=f"admin/{name}.py").destroy(
        after_hooks=[
            RemoveLineFromFile(Path("admin/__init__.py"), command_defaults.admin(name)),
        ],
        **ctx.obj,
    )


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", callback=sanitized_string_callback)
@click.pass_context
def admin_inline(ctx, name):
    """
    Destroy an inline admin model.
    """

    File(name=f"admin/inlines/{name}.py").destroy(
        after_hooks=[
            RemoveLineFromFile(
                Path("admin/inlines/__init__.py"), command_defaults.admin_inline(name)
            ),
        ],
        **ctx.obj,
    )



import unittest
from django_clite.core.field_parser.factory import AttributeFactory


class CallbackTestCase(unittest.TestCase):
    def test_field_parser(self):
        fields = AttributeFactory().parsed_fields(
            [
                "name:char",
                "title:char",
                "user:fk",
                "desc:char",
                "rating:int",
                "owner:fk",
                "total:int",
            ]
        )

        field_names = sorted(fields.keys())
        self.assertEqual(
            ["desc", "name", "owner", "rating", "title", "total", "user"], field_names
        )

        field_values = list(map(lambda x: x.kind, fields.values()))
        self.assertEqual(
            [
                "CharField",
                "CharField",
                "ForeignKey",
                "CharField",
                "IntegerField",
                "ForeignKey",
                "IntegerField",
            ],
            field_values,
        )



import subprocess

from .protocols import Git
from geny.core.decorators.singleton import singleton


@singleton
class GitHandler(Git):
    def initialize(self) -> bool:
        try:
            subprocess.check_output(["git", "init"])
            subprocess.check_output(["git", "add", "--all"])
            subprocess.check_output(["git", "commit", "-m", "feat: initial commit"])
            return True
        except subprocess.CalledProcessError as _:
            pass

        return False

    def add_origin(self, origin: str) -> bool:
        try:
            subprocess.check_output(["git", "remote", "add", "origin", origin])
            return True
        except subprocess.CalledProcessError:
            pass

        return False



import click
import pathlib
import inflection

from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import AddLineToFile, TouchFile
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands import command_defaults
from django_clite.commands.callbacks import sanitized_string_callback


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.option(
    "--skip-import",
    is_flag=True,
    default=False,
    help="Do not import in __init__ module",
)
@click.pass_context
def form(ctx, name, skip_import):
    """
    Generate a form.
    """

    file = File(
        name=f"forms/{name}.py",
        template="form.tpl",
        context={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )

    after_hooks = [TouchFile("forms/__init__.py")]

    if not skip_import:
        after_hooks.append(
            AddLineToFile(
                pathlib.Path("forms/__init__.py"),
                command_defaults.form(name),
                prevent_duplicates=True,
            )
        )

    file.create(after_hooks=after_hooks, **ctx.obj)



import unittest
from django_clite.utils import inflect, sanitized_string

class UtilsTestCase(unittest.TestCase):
    def test_inflection(self):
        singulars = {
            "woman": "women",
            "man": "men",
            "ox": "oxen",
            "car": "cars",
            "child": "children",
        }
        for word, plural in singulars.items():
            self.assertEqual(inflect(word, force_plural=True), plural)

        plurals = {
            "women": "woman",
            "men": "man",
            "oxen": "ox",
            "cars": "car",
            "children": "child",
        }
        for word, singular in plurals.items():
            self.assertEqual(inflect(word, force_singular=True), singular)

    def test_sanitized_string(self):
        string = " Once-Upon a Time "
        self.assertEqual(sanitized_string(string), "once_upon_a_time")

        string = "/my.example/site-1_/"
        self.assertEqual(sanitized_string(string), "my_example_site_1")



import click
import pathlib
import inflection

from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import AddLineToFile, TouchFile
from django_clite.decorators.scope import scoped, Scope
from .. import command_defaults
from django_clite.commands.callbacks import sanitized_string_callback

from .template import template


SUPPORTED_CLASSES = [
    "create",
    "detail",
    "list",
    "update",
]


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.option("--class_", type=click.Choice(SUPPORTED_CLASSES))
@click.option("--full", is_flag=True, help="Create all CRUD views")
@click.option(
    "--skip-templates",
    is_flag=True,
    default=False,
    help="Skip generation of related templates.",
)
@click.option(
    "--skip-import",
    is_flag=True,
    default=False,
    help="Do not import in __init__ module",
)
@click.pass_context
def view(ctx, name, class_, full, skip_templates, skip_import):
    """
    Generate a view function or class.
    """

    classes = SUPPORTED_CLASSES if full else [class_]

    for k in classes:
        file = File(
            name=f"views/{name}{'_' + k if k else ''}.py",
            template=f"views/{k if k else 'view'}.tpl",
            context={
                "name": name,
                "classname": inflection.camelize(name),
                "namespace": inflection.pluralize(name),
                "template_name": f"{name}{'_' + k if k else ''}.hml",
            },
        )

        after_hooks = [TouchFile("views/__init__.py")]

        if not skip_import:
            after_hooks.append(
                AddLineToFile(
                    pathlib.Path("views/__init__.py"),
                    command_defaults.view(name, klass=k),
                    prevent_duplicates=True,
                )
            )

        file.create(after_hooks=after_hooks, **ctx.obj)

    if skip_templates:
        return

    for class_ in classes:
        ctx.invoke(template, name=name, class_=class_)



import click

from pathlib import Path
from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import RemoveLineFromFile
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands.callbacks import sanitized_string_callback
from django_clite.commands import command_defaults


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.pass_context
def manager(ctx, name):
    """
    Destroy a model manager.
    """

    File(name=f"models/managers/{name}.py").destroy(
        after_hooks=[
            RemoveLineFromFile(
                Path("models/managers/__init__.py"), command_defaults.manager(name)
            ),
        ],
        **ctx.obj,
    )



# django_clite:decorators:scope
import os

import click
from enum import Enum

from geny.core.filesystem.finder import core_project_files
from django_clite.constants import ENABLE_FORCE_KEY


class Scope(Enum):
    APP = "app"
    PROJECT = "project"


def scoped(to: Scope):
    def decorator(cmd: click.Command):
        def allowed_to_continue() -> bool:
            django_files = core_project_files(os.getcwd())

            # 1. Probably not inside a django project directory
            if len(django_files) == 0:
                return False

            # 2. Possibly inside a django project, but no app detected
            if to == Scope.APP and django_files.get("apps.py", None):
                return True

            # 3. No django project detected
            matched_project_files = [
                x
                for x in django_files.keys()
                if x in ["manage.py", "wsgi.py", "asgi.py"]
            ]
            if to == Scope.PROJECT and len(matched_project_files) > 0:
                return True

            return False

        class ScopedCommand(click.Command):
            def invoke(self, ctx):
                if ctx.obj.get(ENABLE_FORCE_KEY, False) or allowed_to_continue():
                    super().invoke(ctx)
                    return

                click.echo(
                    f"Command {cmd.name} has '{to.value}' scope but the {to.value} was not detected",
                    err=True,
                )
                raise click.Abort()

        return ScopedCommand(
            add_help_option=cmd.add_help_option,
            callback=cmd.callback,
            context_settings=cmd.context_settings,
            deprecated=cmd.deprecated,
            epilog=cmd.epilog,
            help=cmd.help,
            hidden=cmd.hidden,
            name=cmd.name,
            no_args_is_help=cmd.no_args_is_help,
            options_metavar=cmd.options_metavar,
            params=cmd.params,
            short_help=cmd.short_help,
        )

    return decorator



from django_clite.utils import sanitized_string
from django_clite.core.field_parser.factory import AttributeFactory


def sanitized_string_callback(_ctx, _param, value):
    return sanitized_string(value)


def fields_callback(ctx, _param, values):
    return AttributeFactory().parsed_fields(values)



import unittest
from django_clite.core.field_parser.factory import AttributeFactory


class FieldFactoryTestCase(unittest.TestCase):
    # Numeric

    def test_make_bigint_field(self):
        aliases = ["big", "bigint", "big-int"]
        attr_name = "id"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("BigIntegerField", field.kind)
            self.assertEqual("_('id')", field.field_options(attr_name, model_name))

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    def test_make_decimal_field(self):
        aliases = ["decimal"]
        attr_name = "price"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("DecimalField", field.kind)
            self.assertEqual(
                "_('price'), decimal_places=2, max_digits=8",
                field.field_options(attr_name, model_name),
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    def test_make_float_field(self):
        aliases = ["float"]
        attr_name = "price"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("FloatField", field.kind)
            self.assertEqual("_('price')", field.field_options(attr_name, model_name))

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    def test_make_integer_field(self):
        aliases = ["int", "integer"]
        attr_name = "tracks"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("IntegerField", field.kind)
            self.assertEqual("_('tracks')", field.field_options(attr_name, model_name))

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    # Boolean

    def test_make_boolean_field(self):
        aliases = ["bool", "boolean"]
        attr_name = "is_compilation"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("BooleanField", field.kind)
            self.assertEqual(
                "_('is_compilation'), default=False",
                field.field_options(attr_name, model_name),
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    # Text and Strings

    def test_make_char_field(self):
        aliases = ["char"]
        attr_name = "title"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("CharField", field.kind)
            self.assertEqual(
                "_('title'), max_length=100, blank=False",
                field.field_options(attr_name, model_name),
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    def test_make_slug_field(self):
        aliases = ["slug"]
        attr_name = "slug"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("SlugField", field.kind)
            self.assertEqual(
                "_('slug'), unique=True", field.field_options(attr_name, model_name)
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    def test_make_string_field(self):
        aliases = ["string", "text"]
        attr_name = "description"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("TextField", field.kind)
            self.assertEqual(
                "_('description'), blank=False",
                field.field_options(attr_name, model_name),
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertFalse(field.supports_admin)

    def test_make_uuid_field(self):
        aliases = ["uuid"]
        attr_name = "id"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("UUIDField", field.kind)
            self.assertEqual(
                "_('id'), default=uuid.uuid4, editable=False",
                field.field_options(attr_name, model_name),
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    # Files

    def test_make_file_field(self):
        aliases = ["file"]
        attr_name = "file"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("FileField", field.kind)
            self.assertEqual(
                "_('file'), blank=False, upload_to='uploads/albums/files/'",
                field.field_options(attr_name, model_name),
            )

            self.assertTrue(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertFalse(field.supports_admin)

    def test_make_filepath_field(self):
        aliases = ["filepath", "file-path"]
        attr_name = "file"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("FilePathField", field.kind)
            self.assertEqual(
                "_('file'), blank=True, upload_to='uploads/albums/files/'",
                field.field_options(attr_name, model_name),
            )

            self.assertTrue(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertFalse(field.supports_admin)

    def test_make_image_field(self):
        aliases = ["image", "photo"]
        attr_name = "image"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, "image")

            self.assertEqual("ImageField", field.kind)
            self.assertEqual(
                "_('image'), blank=False, upload_to='uploads/albums/images/'",
                field.field_options(attr_name, model_name),
            )

            self.assertTrue(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertFalse(field.supports_admin)

    # Internet

    def test_make_email_field(self):
        aliases = ["email"]
        attr_name = "email_address"
        model_name = "Person"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("EmailField", field.kind)
            self.assertEqual(
                "_('email_address')", field.field_options(attr_name, model_name)
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    def test_make_url_field(self):
        aliases = ["url"]
        attr_name = "url"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("URLField", field.kind)
            self.assertEqual("_('url')", field.field_options(attr_name, model_name))

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    def test_make_ip_field(self):
        aliases = ["ip", "ipaddress", "ip-address"]
        attr_name = "ip"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("GenericIPAddressField", field.kind)
            self.assertEqual("_('ip')", field.field_options(attr_name, model_name))

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    # Dates and Time

    def test_make_date_field(self):
        aliases = ["date"]
        attr_name = "date"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("DateField", field.kind)
            self.assertEqual(
                "_('date'), auto_now=True", field.field_options(attr_name, model_name)
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    def test_make_datetime_field(self):
        aliases = ["datetime"]
        attr_name = "datetime"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("DateTimeField", field.kind)
            self.assertEqual(
                "_('datetime'), auto_now=True",
                field.field_options(attr_name, model_name),
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    def test_make_duration_field(self):
        aliases = ["duration"]
        attr_name = "duration"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("DurationField", field.kind)
            self.assertEqual(
                "_('duration')", field.field_options(attr_name, model_name)
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    def test_make_time_field(self):
        aliases = ["time"]
        attr_name = "time"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("TimeField", field.kind)
            self.assertEqual(
                "_('time'), auto_now=True", field.field_options(attr_name, model_name)
            )

            self.assertFalse(field.is_media_field)
            self.assertFalse(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    # Relationships
    # =============

    # Foreign Key
    def test_make_foreign_key_field(self):
        aliases = ["belongsto", "belongs-to", "fk", "foreignkey", "foreign-key"]
        attr_name = "singer"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("ForeignKey", field.kind)
            self.assertEqual(
                "Singer, _('singer'), blank=False, on_delete=models.DO_NOTHING, related_name='albums'",
                field.field_options(attr_name, model_name),
            )

            self.assertFalse(field.is_media_field)
            self.assertTrue(field.is_relationship)
            self.assertTrue(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    # One-To-One

    def test_make_one_to_one_field(self):
        aliases = ["one", "hasone", "has-one", "one-to-one"]
        attr_name = "ssn"
        model_name = "Person"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("OneToOneField", field.kind)
            self.assertEqual(
                "Ssn, _('ssn'), blank=False, on_delete=models.CASCADE, primary_key=True, related_name='people'",
                field.field_options(attr_name, model_name),
            )

            self.assertFalse(field.is_media_field)
            self.assertTrue(field.is_relationship)
            self.assertTrue(field.is_fk_relationship)
            self.assertFalse(field.is_many_relationship)
            self.assertTrue(field.supports_admin)

    # Many

    def test_make_many_to_many_field(self):
        aliases = ["hasmany", "has-many", "many", "manytomany", "many-to-many"]
        attr_name = "songs"
        model_name = "Album"

        for alias in aliases:
            field = AttributeFactory().field_options(alias, attr_name)

            self.assertEqual("ManyToManyField", field.kind)
            self.assertEqual(
                "Song, _('songs'), blank=True, on_delete=models.DO_NOTHING, related_name='albums'",
                field.field_options(attr_name, model_name),
            )

            self.assertFalse(field.is_media_field)
            self.assertTrue(field.is_relationship)
            self.assertFalse(field.is_fk_relationship)
            self.assertTrue(field.is_many_relationship)
            self.assertFalse(field.supports_admin)



import click
import logging
from pathlib import Path

from rich.logging import RichHandler

from django_clite.extensions import AliasedAndDiscoverableGroup
from geny.core.filesystem.finder import core_project_files, project_and_app_names
from geny.core.templates.template import TemplateParser

from django_clite.constants import (
    DJANGO_FILES_KEY,
    ENABLE_DRY_RUN_KEY,
    ENABLE_DEBUG_KEY,
    ENABLE_FORCE_KEY,
    ENABLE_VERBOSITY_KEY,
    PROJECT_NAME_KEY,
    APPLICATION_NAME_KEY,
    TEMPLATES_DIRECTORY_ENV_VAR,
)


@click.command(
    cls=AliasedAndDiscoverableGroup, context_settings=dict(ignore_unknown_options=True)
)
@click.option("--debug", is_flag=True, help="Enable debug logs.")
@click.option("--dry", is_flag=True, help="Do not modify the file system.")
@click.option(
    "-f",
    "--force",
    is_flag=True,
    envvar=ENABLE_FORCE_KEY,
    help="Override any conflicting files.",
)
@click.option("--verbose", is_flag=True, help="Enable verbosity.")
@click.option("--project", help="Project name.")
@click.option("--app", help="Application name.")
@click.option(
    "--templates-dir",
    "-t",
    envvar=TEMPLATES_DIRECTORY_ENV_VAR,
    help="Template directory.",
    type=click.Path(),
)
@click.version_option(package_name="django-clite")
@click.pass_context
def main(ctx, debug, dry, force, verbose, project, app, templates_dir):
    """
    django-clite by Leo Neto

    A CLI to handle the creation and management of your Django projects.

    The CLI has some opinions about how your project should be structured in order for it to maximize the
    amount of automatic configuration it can provide you. Since Django itself is highly configurable,
    you are free to bypass conventions of the CLI if you so choose.
    """

    # Note for contributors:
    #
    # Commands are auto-discovered if they are placed under the commands directory.
    # But please be sure to do the following for this to work:
    #   1. Name your package and click command the same.
    #   2. Place your command definition within your package's main.py module
    #   3. Any sub-commands of your command should be added to the top-most command in the package's main.py module.
    #
    #   Access your command like so:
    #   `django-clite my-command my-command-sub-command`
    #
    #   If you would like to skip a plugin/command from being auto-discovered,
    #   simply rename the package by either prepending or appending any number of underscores (_).
    #   Any code contained within the package will be ignored.

    FORMAT = "[DRY] %(message)s" if dry else "%(message)s"

    logging.basicConfig(
        encoding="utf-8",
        level=logging.DEBUG if verbose else logging.INFO,
        format=FORMAT,
        handlers=[
            RichHandler(
                log_time_format="",
                show_path=False,
                show_level=False,
                enable_link_path=True,
                markup=True,
            )
        ],
    )

    django_files = core_project_files()
    project_name, app_name = project_and_app_names(django_files)

    templates = [Path(__file__).resolve().parent / "template_files"]

    if templates_dir is not None:
        templates.append(Path(templates_dir))

    TemplateParser(
        templates_dir=templates,
        context={},
    )

    btx = {
        DJANGO_FILES_KEY: django_files,
        ENABLE_DEBUG_KEY: debug,
        ENABLE_DRY_RUN_KEY: dry,
        ENABLE_FORCE_KEY: force,
        ENABLE_VERBOSITY_KEY: verbose,
        APPLICATION_NAME_KEY: app or app_name,
    }

    # avoids passing an empty project name
    if project or project_name:
        btx[PROJECT_NAME_KEY] = project or project_name

    ctx.ensure_object(dict)
    ctx.obj = btx


if __name__ == "__main__":
    try:
        main()
    except (KeyboardInterrupt, SystemExit) as e:
        click.echo(f"Exited! {repr(e)}")



import pathlib

import click
import inflection

from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import AddLineToFile, TouchFile
from django_clite.commands import command_defaults
from django_clite.commands.callbacks import sanitized_string_callback, fields_callback
from django_clite.decorators.scope import scoped, Scope


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", callback=sanitized_string_callback)
@click.argument("fields", nargs=-1, required=False, callback=fields_callback)
@click.option(
    "--permissions", is_flag=True, help="Add permission stubs to admin model."
)
@click.option(
    "--skip-import",
    is_flag=True,
    default=False,
    help="Do not import in __init__ module",
)
@click.pass_context
def admin(ctx, name, fields, permissions, skip_import):
    """
    Generate an admin model.
    """

    admin_fields = [attr_name for attr_name, v in fields.items() if v.supports_admin]

    file = File(
        name=f"admin/{name}.py",
        template="admin/admin.tpl",
        context={
            "classname": inflection.camelize(name),
            "fields": admin_fields,
            "name": name,
            "permissions": permissions,
        },
    )

    after_hooks = [TouchFile("admin/__init__.py")]

    if not skip_import:
        after_hooks.append(
            AddLineToFile(
                pathlib.Path("admin/__init__.py"),
                command_defaults.admin(name),
                prevent_duplicates=True,
            )
        )

    file.create(after_hooks=after_hooks, **ctx.obj)


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", callback=sanitized_string_callback)
@click.option(
    "--skip-import",
    is_flag=True,
    default=False,
    help="Do not import in __init__ module",
)
@click.pass_context
def admin_inline(ctx, name, skip_import):
    """
    Generate an inline admin model.
    """

    file = File(
        name=f"admin/inlines/{name}.py",
        template="admin/inline.tpl",
        context={
            "classname": inflection.camelize(name),
            "name": name,
        },
    )

    after_hooks = [TouchFile("admin/inlines/__init__.py")]

    if not skip_import:
        after_hooks.append(
            AddLineToFile(
                pathlib.Path("admin/inlines/__init__.py"),
                command_defaults.admin_inline(name),
                prevent_duplicates=True,
            )
        )

    file.create(after_hooks=after_hooks, **ctx.obj)



# django_clite:commands:generate
import click

from django_clite.commands.generate.admin import admin, admin_inline as admin_inline
from django_clite.commands.generate.dockerfile import dockerfile
from django_clite.commands.generate.fixtures import fixture
from django_clite.commands.generate.forms import form
from django_clite.commands.generate.management import management
from django_clite.commands.generate.managers import manager
from django_clite.commands.generate.models import model, scaffold
from django_clite.commands.generate.serializers import serializer
from django_clite.commands.generate.signals import signal
from django_clite.commands.generate.tags import tag
from django_clite.commands.generate.template import template
from django_clite.commands.generate.tests import test
from django_clite.commands.generate.validators import validator
from django_clite.commands.generate.views import view
from django_clite.commands.generate.viewsets import viewset


@click.group()
@click.option("--project", type=click.Path(), help="Project name.")
@click.option("--app", type=click.Path(), help="Application name.")
@click.pass_context
def generate(ctx, project, app):
    """
    Create application resources.
    """

    ctx.ensure_object(dict)


[
    generate.add_command(cmd)
    for cmd in [
        admin,
        admin_inline,
        dockerfile,
        fixture,
        form,
        management,
        manager,
        model,
        scaffold,
        serializer,
        signal,
        tag,
        template,
        test,
        validator,
        view,
        viewset,
    ]
]



from django_clite.extensions.discoverable import DiscoverableGroup


class AliasedAndDiscoverableGroup(DiscoverableGroup):
    """
    Combines support for abbreviated and auto-discoverable commands/plugins
    """

    def __get_command(self, ctx, cmd_name):
        return DiscoverableGroup.get_command(self, ctx, cmd_name)

    def get_command(self, ctx, cmd_name):
        matches = [x for x in self.list_commands(ctx) if x.startswith(cmd_name)]

        if not matches:
            return None

        if len(matches) != 1:
            raise ctx.fail("Too many matches: %s" % ", ".join(sorted(matches)))

        return self.__get_command(ctx, matches[0])



import unittest
from pathlib import Path
from click.testing import CliRunner
from geny.core.filesystem.files import File
from geny.core.filesystem.directories import Directory
from django_clite.commands.new.main import new
from tests import parser as _  # noqa: F401


runner = CliRunner()


class CreatorTestCase(unittest.TestCase):
    def test_new_directory(self):
        with runner.isolated_filesystem():
            name = "store"
            context = {"project": "store", "author": "Leo Neto"}

            inner_proj = Directory(name)

            proj = Directory(
                name=name,
                children=[
                    inner_proj,
                    Directory(name="staticfiles"),
                    Directory(
                        name="templates",
                        children=[
                            File(name="404.html", template="# {{ project }}"),
                            File(name="500.html", template="# {{ project }}"),
                        ],
                    ),
                    File(name="README.md", template="{{project}}/readme.tpl"),
                    File(name="README.txt", template="project/readme.tpl"),
                    File(name="a.py", template="# {{ project }}"),
                    File(name="b.py", content="# content"),
                ],
            )

            proj.create(**context)

            self.assertTrue(proj.path().exists())
            self.assertTrue(proj.path().is_dir())
            self.assertEqual(3, len(proj.dirs))
            self.assertEqual(4, len(proj.files))

            # Template files:
            self.assertEqual(
                "store/readme.tpl\n", proj.files[0].path(proj.path()).read_text()
            )

            with open(Path(name) / "README.txt") as f:
                line = f.readline()
                self.assertEqual("# store\n", line)

            # Template literal
            self.assertEqual("# store\n", proj.files[2].path(proj.path()).read_text())

            # Content literal
            self.assertEqual("# content\n", proj.files[3].path(proj.path()).read_text())

    def test_new_project(self):
        with runner.isolated_filesystem():
            command = "project"
            proj_name = "store"
            res = runner.invoke(new, [command, proj_name])  # noqa

            self.assertEqual(0, res.exit_code)
            self.assertEqual(b"", res.stdout_bytes)

            proj_dir = Path(proj_name)
            self.assertTrue(proj_dir.exists())
            self.assertTrue(proj_dir.is_dir())

            with open(Path(proj_dir) / "README.md") as f:
                line = f.readline()
                self.assertEqual("# store\n", line)

    def test_new_app(self):
        pass  # TODO: Implement test_new_app



import click
import inflection

from geny.core.filesystem.files import File
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands.callbacks import sanitized_string_callback, fields_callback


@scoped(to=Scope.APP)
@click.command()
@click.argument("model", required=True, callback=sanitized_string_callback)
@click.option("--total", default=1, help="Number of fixtures to be created.")
@click.argument("fields", nargs=-1, required=False, callback=fields_callback)
@click.pass_context
def fixture(ctx, model, total, fields):
    """
    Generate model fixtures.
    """

    file = File(
        name=f"fixtures/{model}.json",
        template="fixture.tpl",
        context={
            "total": total,
            "fields": fields,
            "classname": inflection.camelize(model),
        },
    )

    file.create(**ctx.obj)



# # django_clite:commands:new:defaults:app
from geny.core.filesystem.files import File
from geny.core.filesystem.directories import Directory


def generic_package(n: str) -> Directory:
    return Directory(n, children=[File(name="__init__.py", template="shared/init.tpl")])


def new_app(name: str) -> Directory:
    admin = Directory(
        name="admin",
        children=[
            File(name="__init__.py", template="shared/init.tpl"),
            generic_package("actions"),
            generic_package("inlines"),
            generic_package("permissions"),
        ],
    )

    models = Directory(
        "models",
        children=[
            File(name="__init__.py", template="shared/init.tpl"),
            generic_package("managers"),
            generic_package("signals"),
            generic_package("validators"),
        ],
    )

    router = Directory(
        "router",
        children=[
            File(name="__init__.py", template="app/router_init.tpl"),
            File(name="urls.py", template="app/router_urls.tpl"),
        ],
    )

    tests = Directory(
        "tests",
        children=[
            File(
                name="__init__.py",
                content="from .models import *\nfrom .viewsets import *",
            ),
            generic_package("models"),
            generic_package("viewsets"),
        ],
    )

    viewsets = Directory(
        "viewsets",
        children=[
            Directory(name="permissions"),
            Directory(name="mixins"),
            File(name="__init__.py", template="shared/init.tpl"),
        ],
    )

    app = Directory(
        name=name,
        children=[
            admin,
            models,
            router,
            tests,
            viewsets,
            generic_package("fixtures"),
            generic_package("forms"),
            generic_package("middleware"),
            generic_package("migrations"),
            generic_package("serializers"),
            generic_package("tasks"),
            generic_package("templates"),
            generic_package("templatetags"),
            generic_package("views"),
            File(name="__init__.py", template="shared/init.tpl"),
            File(name="apps.py", template="app/apps.tpl"),
            File(name="urls.py", template="app/urls.tpl"),
            File(name="constants.py", content=""),
        ],
    )

    return app


def application_callback(ctx, param, value) -> list[Directory]:
    apps = []

    for name in value:
        app = new_app(name)

        if ctx.params.get("is_package", False):
            app = Directory(
                name=name,
                children=[
                    app,
                    File(name="LICENSE", template="shared/LICENSE.tpl"),
                    File(name="MANIFEST", template="shared/MANIFEST.tpl"),
                    File(name="README.md", template="shared/README.tpl"),
                    File(name="pyproject.toml", template="shared/pyproject.tpl"),
                ],
            )

        apps.append(app)

    return apps



import click

from geny.core.filesystem.files import File
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands.callbacks import sanitized_string_callback


@scoped(to=Scope.APP)
@click.command(name="command")
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.pass_context
def management(ctx, name):
    """
    Generate an application command.
    """

    file = File(
        name=f"management/{name}.py",
        template="management.tpl",
        context={
            "name": name,
        },
    )

    file.create(**ctx.obj)



import click
import pathlib
import inflection

from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import AddLineToFile, TouchFile
from django_clite.core.logger import logger
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands import command_defaults
from django_clite.commands.callbacks import sanitized_string, sanitized_string_callback


SUPPORTED_SCOPES = [
    "model",
    "viewset",
]


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.option("--scope", required=True, type=click.Choice(SUPPORTED_SCOPES))
@click.option("--full", is_flag=True, help=f"Create tests for {SUPPORTED_SCOPES}")
@click.option(
    "--skip-import",
    is_flag=True,
    default=False,
    help="Do not import in __init__ module",
)
@click.pass_context
def test(ctx, name, scope, full, skip_import):
    """
    Generate TestCases.
    """

    if scope and full:
        logger.error("Flags --scope and --full cannot be used simultaneously.")
        raise click.Abort()

    scopes = SUPPORTED_SCOPES if full else [scope]

    for s in scopes:
        filename = f"tests/{inflection.pluralize(s)}/{sanitized_string(name)}_test.py"

        file = File(
            name=filename,
            template=f"{inflection.pluralize(s)}/test.tpl",
            context={
                "name": name,
                "module": name,
                "classname": inflection.camelize(name),
                "namespace": inflection.pluralize(name),
                "scope": "" if scope is None else inflection.pluralize(scope),
            },
        )

        after_hooks = [TouchFile(f"tests/{inflection.pluralize(s)}/__init__.py")]

        if not skip_import:
            after_hooks.append(
                AddLineToFile(
                    pathlib.Path(f"tests/{inflection.pluralize(s)}/__init__.py"),
                    command_defaults.test(name),
                    prevent_duplicates=True,
                )
            )

        file.create(after_hooks=after_hooks, **ctx.obj)



import click

from geny.core.filesystem.files import File
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands.callbacks import sanitized_string_callback


SUPPORTED_CLASSES = [
    "create",
    "detail",
    "list",
    "update",
]


@scoped(to=Scope.APP)
@click.command(context_settings=dict(ignore_unknown_options=True))
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.option("--class_", type=click.Choice(SUPPORTED_CLASSES))
@click.option("--full", is_flag=True, help="Destroy templates for all CRUD operations")
@click.pass_context
def template(ctx, name, class_, full):
    """
    Destroy an html template.
    """

    classes = SUPPORTED_CLASSES if full else [class_]

    for k in classes:
        File(name=f"templates/{name}{'_' + k if k else ''}.html").destroy(**ctx.obj)



import os
import pathlib
import unittest

from click.testing import CliRunner
from django.core.management import call_command
from django.core.management.commands import startapp, startproject

from geny.core.filesystem.filesystem import working_directory
from django_clite.commands.generate.main import generate
from django_clite.commands.destroy.main import destroy

from tests import temp_dir


runner = CliRunner()

app_name = "blogger"


class GeneratorTestCase(unittest.TestCase):
    def setUp(self):
        # Generates a base app wherein all generators should create their modules
        with working_directory(temp_dir):
            call_command(startapp.Command(), app_name, verbosity=0)

    def tearDown(self):
        # Removes files created in setUp()
        for root, dirs, files in os.walk(temp_dir, topdown=False):
            root_path = pathlib.Path(root)
            for name in files:
                if name == ".keep":
                    continue
                (root_path / name).unlink()
            for name in dirs:
                (root_path / name).rmdir()

    def test_require_app_scope_to_run_generators(self):
        with working_directory(temp_dir):
            commands = [
                "admin",
                "fixture",
                "form",
                "manager",
                "model",
                "scaffold",
                "serializer",
                "signal",
                "tag",
                "template",
                "validator",
                "view",
                "viewset",
            ]

            for command in commands:
                with self.subTest(cmd="generate"):
                    res = runner.invoke(generate, [command, "article"])  # noqa
                    self.assertNotEqual(0, res.exit_code)
                    self.assertIn("app was not detected", res.output)

                with self.subTest(cmd="destroy"):
                    res = runner.invoke(destroy, [command, "article"])  # noqa
                    self.assertNotEqual(0, res.exit_code)
                    self.assertIn("app was not detected", res.output)

    def test_require_project_scope_to_generate_dockerfile(self):
        cmd = "dockerfile"

        # Failure

        with self.subTest(name="random directory"):
            with working_directory(temp_dir):
                res = runner.invoke(generate, [cmd])  # noqa
                self.assertNotEqual(0, res.exit_code)
                self.assertIn("project was not detected", res.output)

        with self.subTest(name="app directory"):
            with working_directory(temp_dir / app_name):
                res = runner.invoke(generate, [cmd])  # noqa
                self.assertNotEqual(0, res.exit_code)
                self.assertIn("project was not detected", res.output)

        # Success

        with runner.isolated_filesystem():
            project = "project"
            call_command(startproject.Command(), project, verbosity=0)

            with working_directory(project):
                with self.subTest(name="no flags"):
                        res = runner.invoke(generate, [cmd])  # noqa
                        self.assertEqual(0, res.exit_code)
                        self.assertIn("Dockerfile", os.listdir())

                        # Destroy files
                        d_res = runner.invoke(destroy, [cmd])  # noqa
                        self.assertEqual(0, d_res.exit_code)
                        self.assertNotIn("Dockerfile", os.listdir())

                with self.subTest(name="with flags"):
                    res = runner.invoke(generate, [cmd, "--compose"])  # noqa
                    self.assertEqual(0, res.exit_code)
                    self.assertIn("Dockerfile", os.listdir())
                    self.assertIn("docker-compose.yaml", os.listdir())

                    # Destroy file
                    d_res = runner.invoke(destroy, [cmd, "--compose"])  # noqa

                    self.assertEqual(0, d_res.exit_code)
                    self.assertNotIn("Dockerfile", os.listdir())
                    self.assertNotIn("docker-compose.yaml", os.listdir())

    def test_generate_app_modules(self):
        generators = {
            "admin": "admin",
            "form": "forms",
            "model": "models",
            "serializer": "serializers",
            "viewset": "viewsets",
            "view": "views",
            "tag": "templatetags",
        }

        import_pattern = "from .article import [Aa]rticle([a-zA-Z]+)?$"

        for scope, package in generators.items():
            with self.subTest(scope=scope):
                with working_directory(temp_dir / app_name):
                    g_res = runner.invoke(generate, [scope, "article"])  # noqa

                    self.assertEqual(0, g_res.exit_code)
                    self.assertIn(package, os.listdir())
                    self.assertIn("article.py", os.listdir(package))

                    # Inspect imports
                    with open(pathlib.Path(package) / "__init__.py", "r") as f:
                        self.assertRegex(f.read(), import_pattern)

                    # Destroy file
                    d_res = runner.invoke(destroy, [scope, "article"])  # noqa

                    self.assertEqual(0, d_res.exit_code)
                    self.assertIn(package, os.listdir())
                    self.assertNotIn("article.py", os.listdir(package))

                    # Inspect imports
                    with open(pathlib.Path(package) / "__init__.py", "r") as f:
                        self.assertEqual(f.read(), "")

    def test_generate_model_scoped_app_modules(self):
        generators = {
            "manager":   "managers",
            "signal":    "signals",
            "validator": "validators",
        }

        for scope, package in generators.items():
            with self.subTest(scope=scope):
                with working_directory(temp_dir / app_name):
                    g_res = runner.invoke(generate, [scope, "article"])  # noqa

                    self.assertEqual(0, g_res.exit_code)
                    self.assertIn(package, os.listdir("models"))
                    self.assertIn("article.py", os.listdir(f"models/{package}"))

                    # Inspect imports
                    with open(pathlib.Path(f"models/{package}") / "__init__.py", "r") as f:
                        self.assertRegex(f.read(), 'from .article import [Aa]rticle(_?[a-zA-Z]+)?$')

                    # Destroy file
                    d_res = runner.invoke(destroy, [scope, "article"])  # noqa

                    self.assertEqual(0, d_res.exit_code)
                    self.assertIn(package, os.listdir("models"))
                    self.assertNotIn("article.py", os.listdir(f"models/{package}"))

                    # Inspect imports
                    with open(pathlib.Path(f"models/{package}") / "__init__.py", "r") as f:
                        self.assertEqual(f.read(), "")

    def test_generate_tests(self):
        generators = {
            "model": "models",
            "viewset": "viewsets",
        }

        for scope, package in generators.items():
            with self.subTest(scope=scope):
                with working_directory(temp_dir / app_name):
                    g_res = runner.invoke(generate, ["test", "article", "--scope", scope])  # noqa

                    self.assertEqual(0, g_res.exit_code)
                    self.assertIn(package, os.listdir("tests"))
                    self.assertIn("article_test.py", os.listdir(f"tests/{package}"))

                    # Inspect imports
                    with open(pathlib.Path(f"tests/{package}") / "__init__.py", "r") as f:
                        self.assertRegex(f.read(), 'from .article_test import ArticleTestCase$')

                    # Destroy file
                    d_res = runner.invoke(destroy, ["test", "article", "--scope", scope])  # noqa

                    self.assertEqual(0, d_res.exit_code)
                    self.assertIn(package, os.listdir("tests"))
                    self.assertNotIn("article_test.py", os.listdir(f"tests/{package}"))

                    # Inspect imports
                    with open(pathlib.Path(f"tests/{package}") / "__init__.py", "r") as f:
                        self.assertEqual(f.read(), "")

    def test_admin_inline(self):
        cmd = "admin-inline"

        with working_directory(temp_dir / app_name):
            g_res = runner.invoke(generate, [cmd, "article"])  # noqa

            self.assertEqual(0, g_res.exit_code)
            self.assertIn("admin", os.listdir())
            self.assertIn("inlines", os.listdir("admin"))
            self.assertIn("article.py", os.listdir("admin/inlines"))

            # Inspect imports
            with open(pathlib.Path("admin/inlines") / "__init__.py", "r") as f:
                self.assertRegex(f.read(), 'from .article import ArticleInline$')

            # Destroy file
            d_res = runner.invoke(destroy, [cmd, "article"])  # noqa

            self.assertEqual(0, d_res.exit_code)
            self.assertIn("admin", os.listdir())
            self.assertIn("inlines", os.listdir("admin"))
            self.assertNotIn("article.py", os.listdir("admin/inlines"))

            # Inspect imports
            with open(pathlib.Path("admin/inlines") / "__init__.py", "r") as f:
                self.assertEqual(f.read(), "")

    def test_fixture(self):
        cmd = "fixture"
        package = "fixtures"

        with working_directory(temp_dir / app_name):
            g_res = runner.invoke(generate, [cmd, "article"])  # noqa

            self.assertEqual(0, g_res.exit_code)
            self.assertIn(package, os.listdir())
            self.assertIn("article.json", os.listdir(package))

            # Destroy file
            d_res = runner.invoke(destroy, [cmd, "article"])  # noqa

            self.assertEqual(0, d_res.exit_code)
            self.assertIn(package, os.listdir())
            self.assertNotIn("article.json", os.listdir(package))

    def test_template(self):
        cmd = "template"
        dir_ = "templates"

        with working_directory(temp_dir / app_name):
            g_res = runner.invoke(generate, [cmd, "article", "--full"])  # noqa

            self.assertEqual(0, g_res.exit_code)
            self.assertIn(dir_, os.listdir())
            self.assertEqual(
                [
                    "article_create.html",
                    "article_detail.html",
                    "article_list.html",
                    "article_update.html",
                ],
                sorted(os.listdir(dir_)),
            )

            # Destroy file
            d_res = runner.invoke(destroy, [cmd, "article", "--full"])  # noqa

            self.assertEqual(0, d_res.exit_code)
            self.assertIn(dir_, os.listdir())
            self.assertNotEqual(
                [
                    "article_create.html",
                    "article_detail.html",
                    "article_list.html",
                    "article_update.html",
                ],
                sorted(os.listdir(dir_)),
            )



# django_clite:commands:destroy
import click

from django_clite.commands.destroy.admin import admin, admin_inline as admin_inline
from django_clite.commands.destroy.dockerfile import dockerfile
from django_clite.commands.destroy.fixtures import fixture
from django_clite.commands.destroy.forms import form
from django_clite.commands.destroy.management import management
from django_clite.commands.destroy.managers import manager
from django_clite.commands.destroy.models import model, scaffold
from django_clite.commands.destroy.serializers import serializer
from django_clite.commands.destroy.signals import signal
from django_clite.commands.destroy.tags import tag
from django_clite.commands.destroy.template import template
from django_clite.commands.destroy.tests import test
from django_clite.commands.destroy.validators import validator
from django_clite.commands.destroy.views import view
from django_clite.commands.destroy.viewsets import viewset


@click.group()
@click.pass_context
def destroy(ctx):
    """
    Destroy application resources.
    """

    ctx.ensure_object(dict)


[
    destroy.add_command(cmd)
    for cmd in [
        admin,
        admin_inline,
        dockerfile,
        fixture,
        form,
        management,
        manager,
        model,
        scaffold,
        serializer,
        signal,
        tag,
        template,
        test,
        validator,
        view,
        viewset,
    ]
]



import click

from django_clite.constants import ENABLE_DRY_RUN_KEY
from .defaults.app import application_callback


@click.command()
@click.argument("names", nargs=-1, callback=application_callback)
@click.option("--is-package", is_flag=True, help="Treat as a standalone Python package")
@click.pass_context
def apps(ctx, names, is_package):
    """
    Creates new django apps.

    This is similar to using `django-admin startapp app_name`
    but adds more structure to your app by creating packages for forms, models,
    serializers, tests, templates, views, and viewsets.

    The command can accept multiple apps as arguments. Like so:

        django-clite new apps shop blog forum

    The command above will create all 4 apps two levels within your project's
    directory, i.e. myproject/myproject. The CLI tries to identify where the management module for your
    project is, so it can place your app files in the correct location. This helps with consistency
    as the CLI can infer module/package scopes when performing other automatic configurations.

    As part of the CLI convention, each app is assigned its own `urls.py` file, which can be used to route urls on a
    per-app basis.
    Another convention the CLI adopts is to add a viewsets package to the app's directory by default (for use with DRF).
    Within the viewsets directory, a DRF router is instantiated in `router.py` and its urls added to each app's
    urlpatterns by default.
    """

    if ctx.obj.get(ENABLE_DRY_RUN_KEY):
        return

    for application in names:
        context = dict(ctx.obj)
        context.update({"app": application.name})
        application.create(**context)



import click

from pathlib import Path
from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import RemoveLineFromFile
from django_clite.core.logger import logger
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands import command_defaults
from django_clite.commands.callbacks import sanitized_string_callback


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.option("--api", is_flag=True, help="Destroy only related api resources")
@click.option("--full", is_flag=True, help="Destroy all related resources")
@click.option("--admin", is_flag=True, help="Destroy admin model")
@click.option("--fixtures", is_flag=True, help="Destroy model fixture")
@click.option("--form", is_flag=True, help="Destroy model form")
@click.option("--serializers", is_flag=True, help="Destroy serializers")
@click.option("--templates", is_flag=True, help="Destroy templates")
@click.option("--tests", is_flag=True, help="Destroy tests")
@click.option("--views", is_flag=True, help="Destroy views")
@click.option("--viewsets", is_flag=True, help="Destroy viewsets")
@click.pass_context
def model(
    ctx,
    name,
    api,
    full,
    admin,
    fixtures,
    form,
    serializers,
    templates,
    tests,
    views,
    viewsets,
):
    """
    Destroy a model.
    """

    if api and full:
        logger.error("Flags --api and --full cannot be used simultaneously.")
        raise click.Abort()

    File(name=f"models/{name}.py").destroy(
        after_hooks=[
            RemoveLineFromFile(
                Path("models/__init__.py"), command_defaults.model(name)
            ),
        ],
        **ctx.obj,
    )

    def destroy_related_resources():
        if admin or api or full:
            from .admin import admin as cmd

            ctx.invoke(cmd, name=name)

        if fixtures or api or full:
            from .fixtures import fixture as cmd

            ctx.invoke(cmd, model=name)

        if form or full:
            from .forms import form as cmd

            ctx.invoke(cmd, name=name)

        if serializers or api or full:
            from .serializers import serializer as cmd

            ctx.invoke(cmd, name=name)

        if templates or api or full:
            from .template import template as cmd

            ctx.invoke(cmd, name=name, full=full)

        if tests or api or full:
            from .tests import test as cmd

            ctx.invoke(cmd, name=name, full=full)

        if views or full:
            from .views import view as cmd

            ctx.invoke(cmd, name=name, full=full)

        if viewsets or api or full:
            from .viewsets import viewset as cmd

            ctx.invoke(cmd, name=name)

    destroy_related_resources()


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.pass_context
def scaffold(ctx, name):
    """
    Delete all resources for a given model.
    """

    ctx.invoke(model, name=name, full=True)



import os
from pathlib import Path
from click import MultiCommand
from django_clite.constants import PLUGINS_ENV_VAR
from django_clite.constants import DEV_MODE_ENV_VAR

COMMANDS_FOLDER = Path(__file__).resolve().parent.parent.joinpath("commands")

PLUGINS_FOLDER = os.environ.get(PLUGINS_ENV_VAR, None)


def execute_command(command, ns):
    with open(command) as f:
        code = compile(f.read(), command, "exec")
        eval(code, ns, ns)


class DiscoverableGroup(MultiCommand):
    """
    Adds support for auto-discoverable commands/plugins

    Commands are auto-discovered if they are placed under the COMMAND_FOLDER.

    But please be sure to do the following for this to work:
      1. Name your package and click command the same.
      2. Place your command definition within your package's main.py module
      3. Any sub-commands of your command should be added to the top-most command in the package's main.py module.

      If you would like to skip a plugin/command from being auto-discovered,
      simply rename the package by either prepending or appending any number of underscores (_).
      Any code contained within the package will be ignored.
    """

    COMMAND_PATHS = [COMMANDS_FOLDER]

    def __get_commands(self):
        commands = {}

        if PLUGINS_FOLDER is not None:
            self.COMMAND_PATHS.append(PLUGINS_FOLDER)

        # Core commands + plugins
        for path in self.COMMAND_PATHS:
            c = {
                command_path.rsplit("/", 1)[
                    -1
                ]: command_path  # command_name: command_path
                for command_path, _, files in os.walk(path)
                if "main.py" in files
            }

            commands.update(c)

        return commands

    def list_commands(self, ctx):
        rv = []

        commands = self.__get_commands()

        """
        Only include top-level commands.
        That is, if we find a directory structure like so:
            commands/read/main.py
            commands/read/file/main.py
            commands/read/url/main.py

        We will only add `read` as a command and expect `file` and `url`
        to have been added as sub-commands of `read` already.
        """

        debug_mode = os.getenv(DEV_MODE_ENV_VAR, "false").lower() == "true"

        for func, path in commands.items():
            # Skip packages beginning or ending in underscores (_)
            command = path.split("/")[-1]
            if command not in rv and not (
                command.startswith("_") or command.endswith("_")
            ):
                rv.append(func)

                if debug_mode:
                    print(f"{func} from {path}")

        rv.sort()

        # if debug_mode:
        #     print(rv)

        return rv

    def get_command(self, ctx, name):
        ns = {}
        fn = os.path.join(COMMANDS_FOLDER, name, "main.py")

        try:
            execute_command(fn, ns)
        except FileNotFoundError:
            try:
                pfn = os.path.join(PLUGINS_FOLDER, name, "main.py")
                execute_command(pfn, ns)
            except FileNotFoundError or TypeError:
                pass

        try:
            return ns[name]
        except KeyError:
            # Fail gracefully if command is not found or fails to load
            pass



import inflection
from geny.core.templates.template import TemplateParser


def generic(name: str):
    return TemplateParser().parse_string(
        content="from .{{name}} import {{name}}",
        variables={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )


def admin(name: str):
    return TemplateParser().parse_string(
        content="from .{{name}} import {{classname}}Admin",
        variables={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )


def admin_inline(name: str):
    return TemplateParser().parse_string(
        content="from .{{name}} import {{classname}}Inline",
        variables={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )


def form(name: str):
    return TemplateParser().parse_string(
        content="from .{{name}} import {{classname}}Form",
        variables={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )


def manager(name: str):
    return TemplateParser().parse_string(
        content="from .{{name}} import {{classname}}Manager",
        variables={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )


def model(name: str):
    return TemplateParser().parse_string(
        content="from .{{name}} import {{classname}}",
        variables={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )


def serializer(name: str):
    return TemplateParser().parse_string(
        content="from .{{name}} import {{classname}}Serializer",
        variables={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )


def signal(name: str):
    return generic(name)


def tag(name: str):
    return generic(name)


def test(name: str):
    return TemplateParser().parse_string(
        content="from .{{name}}_test import {{classname}}TestCase",
        variables={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )


def validator(name: str):
    return TemplateParser().parse_string(
        content="from .{{name}} import {{name}}_validator",
        variables={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )


def view(name: str, klass: str):
    return TemplateParser().parse_string(
        content="from .{{name}} import {{classname}}",
        variables={
            "name": f"{name}{'_' + klass if klass else ''}",
            "classname": f"{inflection.camelize(name) + inflection.camelize(klass) + 'View' if klass else name}",
        },
    )


def viewset(name: str):
    return TemplateParser().parse_string(
        content="from .{{name}} import {{classname}}ViewSet",
        variables={
            "name": name,
            "classname": inflection.camelize(name),
        },
    )



import click
import inflection

from geny.core.filesystem.files import File
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands.callbacks import sanitized_string_callback


SUPPORTED_CLASSES = [
    "create",
    "detail",
    "list",
    "update",
]


@scoped(to=Scope.APP)
@click.command(context_settings=dict(ignore_unknown_options=True))
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.option("--class_", type=click.Choice(SUPPORTED_CLASSES))
@click.option("--full", is_flag=True, help="Create templates for all CRUD operations")
@click.pass_context
def template(ctx, name, class_, full):
    """
    Generate an html template.
    """

    classes = SUPPORTED_CLASSES if full else [class_]

    for k in classes:
        file = File(
            name=f"templates/{name}{'_' + k if k else ''}.html",
            template=f"templates/{k if k else 'template'}.tpl",
            context={
                "classname": inflection.camelize(name),
            },
        )

        file.create(**ctx.obj)



import click
from pathlib import Path
from django.core.management import call_command, CommandError
from django.core.management.commands import startproject

from geny.core.filesystem.filesystem import working_directory
from django_clite.commands.callbacks import sanitized_string_callback
from django_clite.constants import ENABLE_DRY_RUN_KEY

from .defaults.project import new_project, project_transformations
from .defaults.app import application_callback
from .app import apps as apps_cmd


@click.command()
@click.argument("name", callback=sanitized_string_callback)
@click.argument("apps", nargs=-1, callback=application_callback)
@click.option(
    "--docker", is_flag=True, help="Render Dockerfile and other docker-related files."
)
@click.option("--github", is_flag=True, help="Render GitHub CI template.")
@click.option("--kubernetes", is_flag=True, help="Render Kubernetes deployment files.")
# @click.option("--dokku", is_flag=True, help="Render Dokku deployment files.")
# @click.option("--heroku", is_flag=True, help="Render Heroku files.")
@click.pass_context
def project(ctx, name, apps, docker, github, kubernetes):  # TODO: dokku, heroku
    """
    Creates a new django project.

    This is similar to using `django-admin startproject project_name` but with some added functionality.
    The command can handle the creation of apps upon creation of any project, you can simply specify the name of
    the apps after the project name:

        django-clite new project myproject app1 app2 app3 ...
    """

    if ctx.obj.get(ENABLE_DRY_RUN_KEY):
        return

    cmd = startproject.Command()

    try:
        call_command(cmd, name, verbosity=0)

        # Generate project files as per CLI conventions
        params = dict(ctx.params)
        params.pop("name")
        params.pop("apps")

        click_ctx = dict(ctx.obj)

        context = {"port": "8080", "workers": 1, "project": name}
        context.update(**click_ctx)
        context.update(**params)

        proj = new_project(name, **context)
        proj.create(**context)

        dir_ = Path(name) / name  # i.e. myproject/myproject
        with working_directory(dir_):
            for t in project_transformations:
                # Customize django-generated files
                t.run()

        # Create nested apps
        with working_directory(name):
            ctx.invoke(apps_cmd, names=apps)

        # TODO: Initialize git repository?

    except CommandError as err:
        click.echo(f"Command returned an error {repr(err)}")
        raise click.Abort()



from .utils import inflect, sanitized_string  # noqa
from .core import field_parser, logger, git  # noqa
from .commands import callbacks, command_defaults, destroy, generate, new  # noqa
from .decorators import scope  # noqa
from .extensions import aliased, combined, discoverable  # noqa
from .cli import main  # noqa



import click

from pathlib import Path
from geny.core.filesystem.files import File
from geny.core.filesystem.transformations import RemoveLineFromFile
from django_clite.decorators.scope import scoped, Scope
from django_clite.commands import command_defaults
from django_clite.commands.callbacks import sanitized_string_callback


@scoped(to=Scope.APP)
@click.command()
@click.argument("name", required=True, callback=sanitized_string_callback)
@click.option("--full", is_flag=True, help="Destroy related files (i.e. TestCases)")
@click.pass_context
def viewset(ctx, name, full):
    """
    Destroy a viewset for a serializable model.
    """

    File(name=f"viewsets/{name}.py").destroy(
        after_hooks=[
            RemoveLineFromFile(
                Path("viewsets/__init__.py"), command_defaults.viewset(name)
            ),
        ],
        **ctx.obj,
    )

    if full:
        from .tests import test as cmd

        ctx.invoke(cmd, name=name, scope="viewset")
